package kieker.webgui.common;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import kieker.analysis.plugin.port.Plugin;

/**
 * @author Nils Christian Ehmke
 */
public class PluginFinder {

    /**
     * This method delivers all classes which are available in the given jar and
     * has the Plugin-Annotation from the Kieker framework.
     *
     * @param url The url for the jar.
     * @return A list containing all available plugin-classes or null, if an
     * exception occured.
     */
    public static List<Class<?>> getAllPluginsWithinJar(final URL url) {
        try {
            /*
             * Get a classloader and make sure that it has the system class
             * loader as parent and that it knows the url.
             */
            final ClassLoader classLoader = new URLClassLoader(new URL[]{url}, ClassLoader.getSystemClassLoader());
            /*
             * Open the jar file and run through all entries within this file.
             */
            final JarFile jarFile = new JarFile(new File(url.getPath()));
            final List<Class<?>> result = new ArrayList<Class<?>>();
            Enumeration<JarEntry> jarEntries = jarFile.entries();
            while (jarEntries.hasMoreElements()) {
                JarEntry jarEntry = (JarEntry) jarEntries.nextElement();
                try {
                    String name = jarEntry.toString();
                    name = name.replace('/', '.');
                    name = name.replace(".class", "");
                    /*
                     * Try to find a class with the same name.
                     */
                    Class<?> c = classLoader.loadClass(name);
                    /*
                     * If it is a class and has the annotation - put it into our
                     * list.
                     */
                    if (c.isAnnotationPresent(Plugin.class)) {
                        result.add(c);
                    }
                } catch (Throwable ex) {
                }
            }
            return result;
        } catch (IOException ex) {
            return null;
        }
    }
}
